/*
 * This file is part of htmltolatex (h2l) a HTML to Latex converter
 * Copyright (C) 2001 Neil Radisch (nradisch@panix.com)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file copying.txt.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
#ifndef ATTRIBUTE_H_
#define ATTRIBUTE_H_

#include <string>
#include <map>

class Attribute
	{
	public:
		typedef enum {UNKNOWN_TYPE,INT_TYPE, PERCENT_TYPE, STRING_TYPE, OFFSET_TYPE, DOUBLE_TYPE} Attribute_t;

	private:
		std::string	mName;
		Attribute_t	mValueType;
		int			mIntVal;
		double		mDoubleVal;
		std::string	mStringVal;

	public:
		Attribute() : mValueType(UNKNOWN_TYPE){}
		Attribute(const std::string &name,Attribute_t at,int val,const std::string &string_rep = "")
			: mName(name),mValueType(at),mIntVal(val),mStringVal(string_rep)
			{
			assert(at == INT_TYPE || at == PERCENT_TYPE || at == OFFSET_TYPE);
			}
		Attribute(const std::string &name,Attribute_t at,double val,const std::string &string_rep = "")
			: mName(name),mValueType(at),mDoubleVal(val),mStringVal(string_rep)
			{
			assert(at == DOUBLE_TYPE);
			}
		Attribute(const std::string &name,Attribute_t at,const std::string &sval)
			: mName(name),mValueType(at),mStringVal(sval) 
			{
			assert(at == STRING_TYPE);
			}

		const std::string&	getName() const 			{ return mName; }
		Attribute_t 		getAttributeType() const 	{ return mValueType; }
		int					getIntVal() const			{ return mIntVal; }
		const std::string&	getStringVal() const		{ return mStringVal; }
		double				getDoubleVal() const		{ return mDoubleVal; }
	};


//typedef std::map<std::string,Attribute> AttributeList_t;

class AttributeList_t: public std::map<std::string,Attribute>
{
public:
	void add_attribute(const Attribute &a)
		{
		(*this)[a.getName()] = a;
		}
};

#endif

