/*
 * This file is part of htmltolatex (h2l) a HTML to Latex converter
 * Copyright (C) 2001 Neil Radisch (nradisch@panix.com)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file copying.txt.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
#include "StyleTree.h"
#include <stdexcept>

void StyleTag::set_from_unparsed_string(const string &s)
{
	if (s.size() == 0) return;
	if (s[0] == '.')
		mClass = s.substr(1,s.size()-1);
	else if (s[0] == '#')
		mId = s.substr(1,s.size()-1);
	else mTag = s;
}

void StyleTag::dump(ostream &o,const string &tabspace) const
{
	o << tabspace << "TAG: " << mTag << " CLASS: " << mClass << " ID: " << mId;
}

	// less than means I'm less specific (more general)
bool StyleTag::moregeneral(const StyleTag &t) const
{
	size_t c1 = 0;
	size_t c2 = 0;
	if ((mTag != t.mTag) && (mTag != "" && t.mTag != ""))
		return false;
	if ((mClass != t.mClass) && (mClass != "" && t.mClass != ""))
		return false;
	if ((mId != t.mId) && (mId != "" && t.mId != ""))
		return false;

	if (mTag != "") c1++;
	if (mClass != "") c1++;
	if (mId != "") c1++;

	if (t.mTag != "") c2++;
	if (t.mClass != "") c2++;
	if (t.mId != "") c2++;

	return c1 < c2;
}



////////////////////////////////////////////////////////////////////

bool StyleEntry::hasParent(const StyleTag &t) const
{ 
	for (StyleTagList_t::const_iterator i = mParents.begin(); i != mParents.end(); ++i)
		{
		if ((*i).mStyleTag == t)
			return true;
		}
	return false;
}

const StyleEntry& StyleEntry::getParent(const string &tag,const string &cls,const string &id) const
{
	StyleTag t(tag,cls,id);

	return getParent(t);
}

const StyleEntry& StyleEntry::getParent(const StyleTag &t) const
{
	for (StyleTagList_t::const_iterator i = mParents.begin(); i != mParents.end(); ++i)
		{
		if ((*i).mStyleTag == t)
			return (*i);
		}
	throw runtime_error("StyleEntry::getParent - not found");
	return *(mParents.end());	// won't ever get here, but have to return something
}

StyleEntry& StyleEntry::getParent(const string &tag,const string &cls,const string &id)
{
	StyleTag t(tag,cls,id);

	return getParent(t);
}

StyleEntry& StyleEntry::getParent(const StyleTag &t)
{
	for (StyleTagList_t::iterator i = mParents.begin(); i != mParents.end(); ++i)
		{
		if ((*i).mStyleTag == t)
			return (*i);
		}
	throw runtime_error("StyleEntry::getParent - not found");
	return *(mParents.end());	// won't ever get here, but have to return something
}

StyleEntry& StyleEntry::addParent(const StyleTag &t)
{
	for (StyleTagList_t::const_iterator i = mParents.begin(); i != mParents.end(); ++i)
		{
		if ((*i).mStyleTag == t)
			throw runtime_error("StyleEntry::addParent - Dupilcate style parent");
		}
	mParents.push_back(StyleEntry(t));
	return mParents.back();
}

void StyleEntry::dump(ostream &o,const string &tabspace) const
{
	mStyleTag.dump(o,tabspace);
	o << endl;
	for (StyleTagList_t::const_iterator i = mParents.begin(); i != mParents.end(); ++i)
		(*i).dump(o,tabspace + "    ");
}

