/*
 * This file is part of htmltolatex (h2l) a HTML to Latex converter
 * Copyright (C) 2001 Neil Radisch (nradisch@panix.com)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file copying.txt.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
#ifndef STYLETREE_H_
#define STYLETREE_H_

#include <vector>
#include "Attribute.h"

class StyleTag
{
private:
	std::string	mTag;
	std::string	mClass;
	std::string	mId;
	AttributeList_t	mAttributes;

	void _assign(const StyleTag &t)
		{
		mTag = t.mTag;
		mClass = t.mClass;
		mId = t.mId;
		mAttributes = t.mAttributes;
		}
public:
	const std::string&	getTag() const { return mTag; }
	const std::string&	getClass() const { return mClass; }
	const std::string&	getId() const { return mId; }
	const AttributeList_t&	getAttributes() const { return mAttributes; }

	void add_attribute(const Attribute &a) { mAttributes[a.getName()] = a;}

	bool operator==(const StyleTag &t) const { return ((mTag == t.mTag) && (mClass == t.mClass) && (mId == t.mId)); }
	StyleTag(){}
	StyleTag(const std::string &tag,const std::string &cls,const std::string &id)
		: mTag(tag),mClass(cls),mId(id){}
	StyleTag(const StyleTag &t) { _assign(t); }
	StyleTag& operator=(const StyleTag &t) { _assign(t); return *this; }

	bool moregeneral(const StyleTag &t) const;
	void setAttributes(const AttributeList_t &alist) { mAttributes = alist; }
	void set_from_unparsed_string(const std::string &s);

	void dump(std::ostream &o,const std::string &tabspace) const;
};

class StyleEntry
{
public:
	typedef std::vector<StyleEntry>	StyleTagList_t;
private:

	StyleTag	mStyleTag;
	StyleTagList_t	mParents;
public:
	StyleEntry(){}
	StyleEntry(const StyleTag &t): mStyleTag(t){}

	bool 				hasParent(const StyleTag &t) const;
	const StyleEntry& 	getParent(const std::string &tag,const std::string &cls,const std::string &id) const;
	const StyleEntry&	getParent(const StyleTag &t) const;
	StyleEntry& 		getParent(const std::string &tag,const std::string &cls,const std::string &id);
	StyleEntry&			getParent(const StyleTag &t);
	StyleEntry& 		addParent(const StyleTag &t);
	const StyleTagList_t&	getParentList() const { return mParents; }
	const StyleTag&			getStyleTag() const { return mStyleTag; }
	void	setAttributes(const AttributeList_t &alist) { mStyleTag.setAttributes(alist); }
	void dump(std::ostream &o,const std::string &tabspace) const;
};
	
#endif
