/*
 * This file is part of htmltolatex (h2l) a HTML to Latex converter
 * Copyright (C) 2001 Neil Radisch (nradisch@panix.com)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file copying.txt.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
#include <iostream>
#include <list>
#include <vector>
#include <string>
#include <stdexcept>
#include "safe_ostrstream.h"

struct tabCell {
	size_t	mRow;
	size_t	mCol;
	size_t	mRowspan;
	size_t	mColspan;
	int		mCellNumber;

	tabCell(size_t row,size_t col,size_t rowspan, size_t colspan,int cellnumber)
		: mRow(row),mCol(col),mRowspan(rowspan),mColspan(colspan),mCellNumber(cellnumber){}

	friend ostream& operator<<(ostream &o,const tabCell &c);
};

class abstractTable: public list<tabCell>
{
public:
	size_t	getRows() const;
	size_t getCols() const;
	void addCell(const tabCell &c);
	void addCell(size_t row,size_t col,size_t rowspan,size_t colspan,int cellnumber);
	bool	existsCellAt(size_t row,size_t col) const;
	bool	existsCellExactlyAt(size_t row,size_t col) const;
	const tabCell&	findCellAt(size_t row,size_t col) const;
	const tabCell&	findCellExactlyAt(size_t row,size_t col) const;
private:
	bool removeCellAt(size_t row,size_t col);
	void vbox(size_t row,size_t col,size_t rowspan);
	void hbox(size_t row,size_t col,size_t colspan);
public:
	void box();

	friend ostream& operator<<(ostream &o,const abstractTable &t);
};



// the same as an abstract table but with different semantics. The column numbers of the
// data have not been offset for rowspan elements.
class HTMLabstractTable: public abstractTable
{
private:
	void box();
public:
	abstractTable makeAbstract() const;
};
