%{
#include "CLexData.h"
#define YYSTYPE	CLexDataPtr

#include "h2l.tab.cpp.h"
#include "CH2L.h"
#include "util.h"
#include <assert.h>

/*
 * This file is part of htmltolatex (h2l) a HTML to Latex converter
 * Copyright (C) 2001 Neil Radisch (nradisch@panix.com)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file copying.txt.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

extern CH2L gH2L;
bool	gEatSpaceAfterTag = true;
bool	gIgnoreNonTagChars = true;
char	gStyleDelimiter = '}';

static std::string	gsIncludeFile;
#define MAX_INCLUDE_DEPTH 10
static YY_BUFFER_STATE gsIncludeStack[MAX_INCLUDE_DEPTH];
static FILE*	gsIncludeFiles[MAX_INCLUDE_DEPTH];
static int gsIncludeStackPtr = 0;
extern bool gMainDataStart;
extern void InitiateMainData();

%}
%option yylineno
%x INTAG
%x INTAGVAL
%x COMMENT
%x SCRIPT
%x SCRIPTCOMMENT
%x EATSPACE
%x DOCTYPE
%x STYLESHEET
%x STYLESHEETVAL
%x STYLESHEETVAL2
%x STYLEBLOCK
%x STYLEBLOCKCOMMENT
%x SKIPSTYLEBLOCK
%x SKIPINLINESTYLES
%x INCLUDE
%x INCLUDEPT2

optwspc	[ \t\n\r\x13]*
notag [^\< \t\n\x13\r\>&]*
attrstring [^\'\"\n\x13\r\t \<\>]*
attrstring1 [^\"]*
attrstring2 [^\']*
stylelocator[^\'\"\n\x13\r\t \<\>\{\}\.\#,]
hexdigit[0-9a-fA-F]
%%

    /*********** Style sheet processing ****************/
{optwspc}\<{optwspc}[sS][tT][yY][lL][eE]{optwspc}\>{optwspc} { 
															if (gH2L.getProcessStyles())
																{
																BEGIN STYLEBLOCK; 
																return STYLEBLOCKBEGIN; 
																}
															else
																{
																BEGIN SKIPSTYLEBLOCK;
																}
															}
<SKIPSTYLEBLOCK>.|\n|\x13										{}
<SKIPSTYLEBLOCK>{optwspc}\<\/{optwspc}[sS][tT][yY][lL][eE]{optwspc}\>{optwspc} { BEGIN 0; }

<STYLEBLOCK>{optwspc}\<\/{optwspc}[sS][tT][yY][lL][eE]{optwspc}\>{optwspc} { BEGIN 0; return STYLEBLOCKEND; }
<STYLEBLOCK>\/\*	{ BEGIN STYLEBLOCKCOMMENT; }
<STYLEBLOCKCOMMENT>.|\n|\x13	{}
<STYLEBLOCKCOMMENT>\*\/	{ BEGIN STYLEBLOCK; }
<STYLEBLOCK>\<!--	{}
<STYLEBLOCK>--\>	{}
<STYLEBLOCK>\.{stylelocator}*	{ yylval = new CLexData; yylval->mTagString = stripstring(lowerstring(yytext)); return STYLE_HTMLTAG; }
<STYLEBLOCK>\#{stylelocator}*	{ yylval = new CLexData; yylval->mTagString = stripstring(lowerstring(yytext)); return STYLE_HTMLTAG; }
<STYLEBLOCK>{stylelocator}*		{ yylval = new CLexData; yylval->mTagString = stripstring(lowerstring(yytext)); return STYLE_HTMLTAG; }
<STYLEBLOCK>[ ]*,[ ]*			{ return STYLE_LOCATOR_LIST_DELIMITER; }
<STYLEBLOCK>[ ]*				{ return STYLE_LOCATOR_DELIMITER; }
<STYLEBLOCK>[ ]*[\{][ ]*		{ gStyleDelimiter = '}'; BEGIN STYLESHEET; return STYLEBEGIN; }
<STYLEBLOCK>.|\n|\x13	{}

<SKIPINLINESTYLES>[\"\']									{ if (gStyleDelimiter == yytext[0]) BEGIN INTAG; }
<SKIPINLINESTYLES>[^\"\']									{}
<INTAG>{optwspc}[sS][tT][yY][lL][eE]{optwspc}={optwspc}\" 	{ 
															gStyleDelimiter = '\"'; 
															if (gH2L.getProcessStyles())
																{
																BEGIN STYLESHEET; 
																return STYLEBEGIN; 
																}
															else
																{
																BEGIN SKIPINLINESTYLES;
																}
															}
<INTAG>{optwspc}[sS][tT][yY][lL][eE]{optwspc}={optwspc}\' 	{ 
															gStyleDelimiter = '\''; 
															if (gH2L.getProcessStyles())
																{
																BEGIN STYLESHEET; 
																return STYLEBEGIN; 
																}
															else
																{
																BEGIN SKIPINLINESTYLES;
																}
															}

<STYLESHEET>[\"\'\}]										{ 
															if (gStyleDelimiter == yytext[0]) 
																{ 
																if (yytext[0] == '}')
																	BEGIN STYLEBLOCK;
																else
																	BEGIN INTAG; 
																return STYLEEND; 
																} 
															}
<STYLESHEET>[\}]{optwspc}									{ 
															if (gStyleDelimiter == yytext[0]) 
																{ 
																BEGIN STYLEBLOCK;
																return STYLEEND; 
																} 
															}
<STYLESHEET>{optwspc}[a-zA-Z\-]+{optwspc}:{optwspc}			{ 
															yylval = new CLexData; 
															yylval->mTagString = stripstring(removecolonclause(lowerstring(string(yytext).substr(0,strlen(yytext))))); 
															if (gStyleDelimiter == '}')
																{ BEGIN STYLESHEETVAL2; }
															else
																{ BEGIN STYLESHEETVAL; }
															return ATTRIBUTE_NAME_VALUE;
															}

<STYLESHEETVAL>[^;\'\"\}]*									{ yylval = new CLexData; yylval->mStringVal = stripstring(yytext); return ATTRIBUTE_VALUE_STRING; }
<STYLESHEETVAL2>[^;\}]*										{ yylval = new CLexData; yylval->mStringVal = stripstring(yytext); return ATTRIBUTE_VALUE_STRING; }
<STYLESHEETVAL>[;\'\"\}]{optwspc}							{ 
															if (yytext[0] == ';') 
																{ 
																BEGIN STYLESHEET; 
																} 
															else if (yytext[0] == gStyleDelimiter) 
																{ 
																if (gStyleDelimiter == '\'' || gStyleDelimiter == '"') 
																	{BEGIN INTAG;} 
																else 
																	{BEGIN STYLEBLOCK;} 
																return STYLEEND; 
																}
															}
<STYLESHEETVAL2>[;\}]{optwspc}								{
															if (yytext[0] == ';') 
																{ 
																BEGIN STYLESHEET; 
																} 
															else
																{ 
																BEGIN STYLEBLOCK; 
																return STYLEEND; 
																}
															}


    /**************************************************/


<EATSPACE>{optwspc}								{ /* gEatSpaceAfterTag = false; BEGIN 0; yylval = 0; */ }
<EATSPACE>.										{ unput(yytext[0]); BEGIN 0; yylval = 0; if (yytext[0] != '<') gEatSpaceAfterTag = false; }
<INTAG>{optwspc}\>								{ yylval = 0; if (gEatSpaceAfterTag) { BEGIN EATSPACE; } else { BEGIN 0;}  return TAG_CLOSE; }
    /* <INTAGVAL>{optwspc}\>							{ yylval = 0; if (gEatSpaceAfterTag) { BEGIN EATSPACE; } else { BEGIN 0;}  return TAG_CLOSE; } */

<INTAG>{optwspc}[a-zA-Z][a-zA-Z0-9\-]*{optwspc} 				{ yylval = new CLexData; yylval->mTagString = stripstring(lowerstring(string(yytext).substr(0,strlen(yytext)))); return ATTRIBUTE_NAME;}
<INTAG>{optwspc}[a-zA-Z][a-zA-Z0-9\-]*{optwspc}={optwspc} 	{ yylval = new CLexData; yylval->mTagString = stripstring(removeequalclause(lowerstring(string(yytext).substr(0,strlen(yytext))))); BEGIN INTAGVAL; return ATTRIBUTE_NAME_VALUE;}

<INTAG>\"									{ /* screwup check.. extra quotes inside a tag */}
<INTAG>[_\+]									{}

<INTAGVAL>[0-9]+							{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = yytext; yylval->mIntVal = atoi(yytext); return ATTRIBUTE_VALUE_INT; }
<INTAGVAL>[-+][0-9]+						{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = yytext; yylval->mIntVal = atoi(yytext); return ATTRIBUTE_VALUE_OFFSET; }
<INTAGVAL>\"[0-9]+\"						{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = string(yytext).substr(1,strlen(yytext)-2); yylval->mIntVal = atoi(&(yytext[1])); return ATTRIBUTE_VALUE_INT; }
<INTAGVAL>\"[-+][0-9]+\"					{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = string(yytext).substr(1,strlen(yytext)-2); yylval->mIntVal = atoi(&(yytext[1])); return ATTRIBUTE_VALUE_OFFSET; }
<INTAGVAL>\'[0-9]+\'						{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = string(yytext).substr(1,strlen(yytext)-2); yylval->mIntVal = atoi(&(yytext[1])); return ATTRIBUTE_VALUE_INT; }
<INTAGVAL>\'[-+][0-9]+\'					{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = string(yytext).substr(1,strlen(yytext)-2); yylval->mIntVal = atoi(&(yytext[1])); return ATTRIBUTE_VALUE_OFFSET; }
<INTAGVAL>[0-9]+%							{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = yytext; yylval->mIntVal = atoi(&(yytext[0])); return ATTRIBUTE_VALUE_PERCENT; }
<INTAGVAL>\"[0-9]+%\"						{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = string(yytext).substr(1,strlen(yytext)-2); yylval->mIntVal = atoi(&(yytext[1])); return ATTRIBUTE_VALUE_PERCENT; }
<INTAGVAL>\'[0-9]+%\'						{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = string(yytext).substr(1,strlen(yytext)-2); yylval->mIntVal = atoi(&(yytext[1])); return ATTRIBUTE_VALUE_PERCENT; }
<INTAGVAL>[a-zA-Z#_]{attrstring}				{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = yytext; return ATTRIBUTE_VALUE_STRING; }
<INTAGVAL>\"{attrstring1}\"					{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = string(yytext).substr(1,strlen(yytext)-2); return ATTRIBUTE_VALUE_STRING; }
<INTAGVAL>\'{attrstring2}\'					{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = string(yytext).substr(1,strlen(yytext)-2); return ATTRIBUTE_VALUE_STRING; }
<INTAGVAL>{hexdigit}{6}						{ BEGIN INTAG; yylval = new CLexData; yylval->mStringVal = yytext; return ATTRIBUTE_VALUE_STRING; }
<INTAGVAL>\>								{ BEGIN INTAG; unput(yytext[0]); yylval = new CLexData; yylval->mStringVal = ""; return ATTRIBUTE_VALUE_STRING; }


\<!--					{ BEGIN COMMENT; yylval = 0; }
<COMMENT>--\>			{ BEGIN EATSPACE; /* was BEGIN 0; */  yylval = 0; }
<COMMENT>.|\n|\x13			{ yylval = 0; }

\<![dD][oO][cC][tT][yY][pP][eE]				{ BEGIN DOCTYPE; yylval = 0; }
<DOCTYPE>\>									{ BEGIN 0; yylval = 0; }
<DOCTYPE>.|\n|\x13							{ yylval = 0; }

<SCRIPT>{optwspc}\<\/[sS][cC][rR][iI][pP][tT]\>{optwspc}	{ BEGIN 0; }
<SCRIPTCOMMENT>{optwspc}\<\/[sS][cC][rR][iI][pP][tT]\>{optwspc}	{ BEGIN 0; /* deals with     // --></script> constructs */ }
<SCRIPT>\"{attrstring1}\"									{}
<SCRIPT>\'{attrstring2}\'									{}
<SCRIPT>\/\/												{ BEGIN SCRIPTCOMMENT; }
<SCRIPT>.|\n|\x13												{}
<SCRIPTCOMMENT>.											{}
<SCRIPTCOMMENT>\n|\x13											{ BEGIN SCRIPT; }

{optwspc}\<[hH][tT][mM][lL]					{ yylval = 0; BEGIN INTAG; return TAG_HTML_START; }
{optwspc}\<\/[hH][tT][mM][lL]{optwspc}\>(.|\n|\x13)*	{ yylval = 0; return TAG_HTML_END; gIgnoreNonTagChars=true;}
{optwspc}\<[hH][eE][aA][dD]					{ yylval = 0; BEGIN INTAG; return TAG_HEAD_START; }
{optwspc}\<\/[hH][eE][aA][dD]{optwspc}\>{optwspc}	{ yylval = 0; return TAG_HEAD_END; }
{optwspc}\<[tT][iI][tT][lL][eE]				{ yylval = 0; BEGIN INTAG; return TAG_TITLE_START; }
{optwspc}\<\/[tT][iI][tT][lL][eE]{optwspc}\>{optwspc} { yylval = 0; return TAG_TITLE_END; }
{optwspc}\<[sS][cC][rR][iI][pP][tT]			{ BEGIN SCRIPT; }

\<\/[bB][rR]{optwspc}\>	{ /* wierd end br tag I've seen. Ignore it*/  }
						
\<[bB][rR][][/] { 				// wierd br tag I saw at php.net
							yylval = new CLexData; 
							yylval->mTagString = "br";
							gEatSpaceAfterTag = gH2L.tag_ignores_spaces(yylval->mTagString); 
							BEGIN INTAG; return SINGLETAG_START; 
						}

\<[lL][iI][nN][kK]{optwspc}		{ BEGIN INCLUDE; gsIncludeFile = ""; }

\<[a-zA-z][a-zA-Z0-9\-]*	{ 
						bool last_eat_space = gEatSpaceAfterTag;
						yylval = new CLexData; 
						std::string t = lowerstring(string(yytext).substr(1,strlen(yytext)-1));
						yylval->mTagString = t;
						gEatSpaceAfterTag = gH2L.tag_ignores_spaces(yylval->mTagString) || last_eat_space; 
						BEGIN INTAG;
						return (gH2L.is_single_tag(t) ? SINGLETAG_START : BLOCKTAG_START);
						}
\<\/[a-zA-Z][a-zA-Z0-9\-]*	{ 
							bool last_eat_space = gEatSpaceAfterTag;
							yylval = new CLexData; 
							std::string t = lowerstring(string(yytext).substr(1,strlen(yytext)-1));
							if (gH2L.is_single_tag(t))
								{
								yylval->mTagString = t;
								gEatSpaceAfterTag = gH2L.tag_ignores_spaces(yylval->mTagString) || last_eat_space; 
								BEGIN INTAG; 
								return SINGLETAG_START; 
								}
							else
								{
								yylval->mTagString = lowerstring(string(yytext).substr(2,strlen(yytext)-2)); 
								gEatSpaceAfterTag = gH2L.tag_ignores_spaces(yylval->mTagString) || last_eat_space; 
								BEGIN INTAG; 
								return BLOCKTAG_END; 
								}
							}


<INCLUDE>{optwspc}[hH][rR][eE][fF]{optwspc}={optwspc} { if (gH2L.getProcessStyles()) {BEGIN INCLUDEPT2;} }
<INCLUDE>{optwspc}\>{optwspc}			{ 
										if (gsIncludeStackPtr >= MAX_INCLUDE_DEPTH)
											{
											cerr << "Includes nested too deeply" << endl;
											}
										else if (gsIncludeFile != "")
											{
											gsIncludeStack[gsIncludeStackPtr] = YY_CURRENT_BUFFER;
											gsIncludeFiles[gsIncludeStackPtr] = yyin;
											yyin = popen(string("fetch -o - " + gsIncludeFile).c_str(),"r");
											if (!yyin)
												{
												cerr << "Could not open linked file " << gsIncludeFile << endl;
												yy_switch_to_buffer(gsIncludeStack[gsIncludeStackPtr]);
												yyin = gsIncludeFiles[gsIncludeStackPtr];
												}
											else
												{
												yy_switch_to_buffer(yy_create_buffer(yyin,YY_BUF_SIZE));
												gsIncludeStackPtr++;
												BEGIN STYLEBLOCK; return STYLEBLOCKBEGIN;
												}
											}
										BEGIN 0; 
										}
<INCLUDE>.|\n|\x13							{}
<INCLUDEPT2>{optwspc}\>{optwspc}		{ BEGIN 0; }
<INCLUDEPT2>{optwspc}\"{attrstring1}\"	{ 
										gsIncludeFile = stripstring(yytext);
										gsIncludeFile = gsIncludeFile.substr(1,gsIncludeFile.size() - 2);
										if (gsIncludeFile.find(".css") == string::npos) gsIncludeFile = "";
										BEGIN INCLUDE; 
										}
<INCLUDEPT2>{optwspc}\'{attrstring2}\'	{ 
										gsIncludeFile = stripstring(yytext);
										gsIncludeFile = gsIncludeFile.substr(1,gsIncludeFile.size() - 2);
										BEGIN INCLUDE; 
										}
<INCLUDEPT2>{optwspc}{attrstring}		{ 
										gsIncludeFile = stripstring(yytext);
										BEGIN INCLUDE; 
										}


<<EOF>> 		{ 
				if (gsIncludeStackPtr <= 0)
					{
					yylval = 0; 
					return 0; 
					}
				else
					{
					if (yyin) pclose(yyin);
					yy_delete_buffer(YY_CURRENT_BUFFER);					
					yy_switch_to_buffer(gsIncludeStack[--gsIncludeStackPtr]);
					BEGIN 0; 
					return STYLEBLOCKEND;
					}
				}
[ \t\n\r]+		{ if (gIgnoreNonTagChars) { yylval = 0; } else {yylval = new CLexData; if (gH2L.verbatim_mode()) yylval->mStringVal = yytext; else yylval->mStringVal=" "; return OTHER; }}
&[a-z]+;		{ 
				if (!gMainDataStart) 
					{
					InitiateMainData();
					gIgnoreNonTagChars = false;
					}
				if (gIgnoreNonTagChars) 
					{ yylval = 0; } 
				else 
					{yylval = new CLexData; yylval->mStringVal = lookup_special(string(yytext).substr(1,strlen(yytext)-2)); return OTHER; }
				}
&#[0-9]{1,3};	{ 
				if (!gMainDataStart)
					{
					InitiateMainData();
					gIgnoreNonTagChars = false;
					}
				if (gIgnoreNonTagChars) { yylval = 0; } else {yylval = new CLexData; char *p = &(yytext[2]); while((*p) == '0') p++; yylval->mStringVal = (unsigned char)atoi(p); return OTHER; }
				}
{notag}			{ 
				if (!gMainDataStart)
					{
					InitiateMainData();
					gIgnoreNonTagChars = false;
					}
				if (gIgnoreNonTagChars) { yylval = 0; } else {yylval = new CLexData; yylval->mStringVal = yytext;  return OTHER; }
				}
.				{ 
				if (!gMainDataStart)
					{
					InitiateMainData();
					gIgnoreNonTagChars = false;
					}
				if (gIgnoreNonTagChars) { yylval = 0; } else {yylval = new CLexData; yylval->mStringVal = yytext[0]; return OTHER; }
				}
\x13			{}


%%

int yyerror(const char *s)
{
	cerr << "Error: " << s << "[" << yytext << "] line " << yylineno << endl;

	return 0;
}



