%{

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include "CH2L.h"
#include "CLexData.h"
#include "util.h"
#include <assert.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "StyleTree.h"

/*
 * This file is part of htmltolatex (h2l) a HTML to Latex converter
 * Copyright (C) 2001 Neil Radisch (nradisch@panix.com)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file copying.txt.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

#define YYSTYPE	CLexDataPtr

int yyerror(const char *s);
int yylex(void);
void yyrestart(FILE* f);
void warning(const char *s) { cerr << s << endl; }
CH2L gH2L;
extern bool	gIgnoreNonTagChars;

static StyleTag gsCurrentStyleTag;
static vector<StyleTag> gsCurrentStyleTagList;
static vector< vector<StyleTag> > gsCurrentStyleTagListCollection;
static bool gsStyleBlock = false;
static AttributeList_t	gsStyleAttributes;
bool gMainDataStart = false;

void InitiateMainData()
{
	gH2L.reset_attribute_stack(); 
	gH2L.latex_preheader(); 
	gMainDataStart = true;
}

%}

%token OTHER
%token BLOCKTAG_START BLOCKTAG_END
%token SINGLETAG_START
%token TAG_CLOSE
%token TAG_HTML_START TAG_HTML_END
%token TAG_HEAD_START TAG_HEAD_END
%token TAG_TITLE_START TAG_TITLE_END

%token ATTRIBUTE_VALUE_INT ATTRIBUTE_VALUE_PERCENT ATTRIBUTE_VALUE_STRING ATTRIBUTE_VALUE_OFFSET
%token ATTRIBUTE_NAME ATTRIBUTE_NAME_VALUE

%token STYLEBEGIN
%token STYLEEND

%token STYLEBLOCKBEGIN
%token STYLEBLOCKEND
%token STYLE_HTMLTAG
%token STYLE_LOCATOR_LIST_DELIMITER
%token STYLE_LOCATOR_DELIMITER

%%

document: 	html_block	{ if (gMainDataStart) { gH2L.html_close(); gMainDataStart = false; } gIgnoreNonTagChars = true;} 
	|		html_content { if (gMainDataStart) { gH2L.html_close(); gMainDataStart = false; } gIgnoreNonTagChars = true;}
	;


text:		OTHER	{ $$ = $1; $1 = 0; }
	| text OTHER	{ $$ = new CLexData; $$->mStringVal = $1->mStringVal + $2->mStringVal; delete $1; $1 = 0; delete $2; $2 = 0;}
	;

html_block_init:	TAG_HTML_START	{ 
									if (!gMainDataStart) 
										InitiateMainData();  
									gIgnoreNonTagChars = false; 
									} // |				text TAG_HTML_START	{ delete $1; $1 = 0; gH2L.reset_attribute_stack(); gH2L.latex_preheader(); }
	;

html_block_start:	html_block_init TAG_CLOSE
	|				html_block_init attribute_list TAG_CLOSE
	;
html_block_end:		TAG_HTML_END {	gH2L.html_close(); gMainDataStart = false; } 
	;
html_block:			html_block_start html_content html_block_end
	|				html_block_start html_block_end
	;

html_extra_elements:	title_block
	|					style_block
	|					head_block
	;

html_extra_elements_list:	html_extra_elements
	|						html_extra_elements_list html_extra_elements
	;

html_content:	html_extra_elements_list
	|			html_extra_elements_list element_list
	|			element_list
	|			html_extra_elements_list element_list html_extra_elements_list
	;


head_element_list:	head_element
	|				head_element_list head_element
	;

head_element:	OTHER					{ delete $1; $1 = 0; }
	|			title_block				{ gIgnoreNonTagChars = true; }
	|			tag_block				{ delete $1; $1 = 0; }
	|			single_tag				{ 
										delete $1; $1 = 0; 
										}
	|			style_block
	;

head_block_init:	TAG_HEAD_START	{ if (!gMainDataStart) InitiateMainData(); }
head_block_start:	head_block_init TAG_CLOSE				{ gIgnoreNonTagChars = true; }
	|				head_block_init  attribute_list TAG_CLOSE	{ gIgnoreNonTagChars = true; }
	;
head_block_end:		TAG_HEAD_END	{ gIgnoreNonTagChars = false; }
	;
head_block: 		head_block_start head_block_end
	|				head_block_start head_element_list head_block_end
	;

title_block_start:	TAG_TITLE_START { gIgnoreNonTagChars = false; } TAG_CLOSE
	|				TAG_TITLE_START { gIgnoreNonTagChars = false; } attribute_list TAG_CLOSE
	;
title_block_end:	TAG_TITLE_END
	;
title_block_content:		{ $$ = new CLexData; }
	|				text	{ $$ = $1; $1 = 0; }
	;
title_block:	title_block_start title_block_content title_block_end { gH2L.set_title($2->mStringVal); delete $2; $2 = 0;}
	;

	
single_tag_init:		SINGLETAG_START		{
											if (!gMainDataStart)
												{ 
												InitiateMainData();
												gIgnoreNonTagChars = false;
												}
											$$ = $1;
											gH2L.execute_implied_closes($1->mTagString);
											gH2L.new_attributes($1->mTagString);
											$1 = 0;
											}
	;
single_tag:			single_tag_init TAG_CLOSE					{ $$ = $1; gH2L.tag_make($1->mTagString); $1 = 0;}
	|				single_tag_init attribute_list TAG_CLOSE	{ $$ = $1; gH2L.tag_make($1->mTagString); $1 = 0;}
	;


tag_block_init:			BLOCKTAG_START								{
																	if (!gMainDataStart)
																		{
																		InitiateMainData();
																		gIgnoreNonTagChars = false;
																		}
																	$$ = $1;
																	gH2L.execute_implied_closes($1->mTagString);
																	gH2L.new_attributes($1->mTagString); 
																	$1 = 0;
																	}
	;
tag_block_start:	tag_block_init TAG_CLOSE					{ $$ = $1; gH2L.tag_start($1->mTagString); $1 = 0;}
	|				tag_block_init attribute_list TAG_CLOSE		{ $$ = $1; gH2L.tag_start($1->mTagString); $1 = 0;}
	;
tag_block_end:		BLOCKTAG_END	TAG_CLOSE					{ $$ = $1; $1 = 0; }
	;
tag_block:			tag_block_start element_list tag_block_end	{ 	$$ = $1; 
																	bool mismatch = ($1->mTagString != $3->mTagString);
																	if (mismatch)
																		warning(string(string("Mismatched tags ") + $1->mTagString + " " + $3->mTagString).c_str()); 
																	gH2L.tag_end($1->mTagString); 																	
																	if (mismatch)
																		gH2L.tag_end($3->mTagString);
																	delete $3; $1 = 0; $3 = 0;
																}
	|				tag_block_start tag_block_end				{ 
																	$$ = $1; 
																	bool mismatch = ($1->mTagString != $2->mTagString);
																	if (mismatch)
																		warning(string(string("Mismatched tags ") + $1->mTagString + " " + $2->mTagString).c_str()); 
																	gH2L.tag_end($1->mTagString); 
																	if (mismatch)
																		gH2L.tag_end($2->mTagString);
																	delete $2; $1 = 0; $2 = 0;
																}	
	;

element_list:	element
	|			element_list element
	|			error
	;

element:	OTHER					{ gH2L.accumulate_text($1->mStringVal); delete $1; $1 = 0;}
	|		tag_block				{ delete $1; $1 = 0; }
	|		single_tag				{ delete $1; $1 = 0; }
	;


style_attribute_list:	STYLEBEGIN attribute_list STYLEEND
	;

attribute_list:	attribute
	|			style_attribute_list
	|			attribute_list attribute
	|			attribute_list style_attribute_list
	;

attribute:	ATTRIBUTE_NAME						{ 
												Attribute a($1->mTagString,Attribute::INT_TYPE,1,"1");
												if (gsStyleBlock) 
													gsStyleAttributes.add_attribute(a); 
												else
													gH2L.add_attribute(a);
												delete $1; $1 = 0; 
												}
	|
			ATTRIBUTE_NAME_VALUE ATTRIBUTE_VALUE_INT	{ 
														Attribute a($1->mTagString,Attribute::INT_TYPE, $2->mIntVal, $2->mStringVal);
														if (gsStyleBlock)
															gsStyleAttributes.add_attribute(a);
														else
															gH2L.add_attribute(a); 
														delete $1; $1 = 0; delete $2; $2 = 0;
														}
	|		ATTRIBUTE_NAME_VALUE ATTRIBUTE_VALUE_OFFSET	{ 
														Attribute a($1->mTagString,Attribute::OFFSET_TYPE, $2->mIntVal, $2->mStringVal);
														if (gsStyleBlock)
															gsStyleAttributes.add_attribute(a);
														else
															gH2L.add_attribute(a); 
														delete $1; $1 = 0; delete $2; $2 = 0;
														}
	|		ATTRIBUTE_NAME_VALUE ATTRIBUTE_VALUE_PERCENT	{ 
															Attribute a($1->mTagString,Attribute::PERCENT_TYPE, $2->mIntVal, $2->mStringVal);
															if (gsStyleBlock)
																gsStyleAttributes.add_attribute(a);
															else
																gH2L.add_attribute(a); 
															delete $1; $1 = 0; delete $2; $2 = 0;
															}
	|		ATTRIBUTE_NAME_VALUE ATTRIBUTE_VALUE_STRING	{ 
														Attribute a($1->mTagString,Attribute::STRING_TYPE, $2->mStringVal);
														if (gsStyleBlock)
															gsStyleAttributes.add_attribute(a);
														else
															gH2L.add_attribute(a); 
														delete $1; $1 = 0; delete $2; $2 = 0; 
														}
	;


style_locator_tag:	STYLE_HTMLTAG	{ gsCurrentStyleTag = StyleTag(); gsCurrentStyleTag.set_from_unparsed_string($1->mTagString);  delete $1; $1 = 0; }
	|				style_locator_tag STYLE_HTMLTAG { gsCurrentStyleTag.set_from_unparsed_string($2->mTagString); delete $2; $2 = 0; }
	;

style_locator:	style_locator_tag	{ gsCurrentStyleTagList = vector<StyleTag>(); gsCurrentStyleTagList.push_back(gsCurrentStyleTag); }
	|			style_locator STYLE_LOCATOR_DELIMITER style_locator_tag { gsCurrentStyleTagList.push_back(gsCurrentStyleTag);}
	;

style_locator_list:	style_locator { gsCurrentStyleTagListCollection = vector< vector<StyleTag> >(); gsCurrentStyleTagListCollection.push_back(gsCurrentStyleTagList); }
	|				style_locator_list STYLE_LOCATOR_LIST_DELIMITER style_locator { gsCurrentStyleTagListCollection.push_back(gsCurrentStyleTagList); }
	;

style_entry:	style_locator_list style_attribute_list { gH2L.add_style_sheet_entry(gsCurrentStyleTagListCollection,gsStyleAttributes); gsStyleAttributes = AttributeList_t(); }
	;

style_block_contents:	style_entry
	|					style_block_contents style_entry
	;

style_block:	STYLEBLOCKBEGIN STYLEBLOCKEND	{ gsStyleBlock = false; }
	|			STYLEBLOCKBEGIN { gsStyleBlock = true; }  style_block_contents STYLEBLOCKEND { gsStyleBlock = false; }
	;


%%


extern FILE *yyin;

int main(int argc,char *argv[])
{
	int	spider_limit = -1;
	bool nospider = false;
	FILE *newfile = 0;
	bool nostdin = false;
	std::string url;
	std::string server,path,file;

	// process args
	for (int x = 1; x < argc; ++x)
		{
		string s = argv[x];
		if (s == "-h" || s == "-help")
			{
			cout << "-                            Data NOT coming from stdin. Fetch it based on -url parameter" << endl;
			cout << "-allowfilefetch              Allow file:/// URL type (security risk)" << endl;
			cout << "-data <path>                 Path to data file (nmr.tex - should end in slash)" << endl;
			cout << "-tmp <path>                  Path for temp files (should end in slash). Default is /var/tmp/" << endl;
			cout << "-paperwidth <dimspec>        Width of paper" << endl;
			cout << "-paperheight <dimspec>       Height of paper" << endl;
			cout << "-hmargin <dimspec>           Horizontal margin" << endl;
			cout << "-vmargin <dimspec>           Vertical margin" << endl;
			cout << "-nostyles                    Ignore all style sheet commands (embedded or otherwise)" << endl;
			cout << "-inlinetables                Do not use external file for table dimensions (limits size of tables)" << endl;
			cout << "-nohyperlinks                Do not embed PDF hyperlinks" << endl;
			cout << "-notables                    Ignore table tags and just dump the cell data as paragraphs" << endl;
			cout << "-nospider                    Do not follow local hyperlinks" << endl;
			cout << "-noimages                    Do not fetch images" << endl;
			cout << "-spiderlimit <cnt>           Max pages to spider. -1 means no limit (default)" << endl;
			cout << "-url <url>                   Base url used for hyperlinks and images" << endl;
			cout << "-nowhitecolortables          Do not use Latex color tables for white tables. Plain black and white tables might" << endl;
			cout << "                             look better with this option, but special effects that use a white table over a" << endl;
			cout << "                             colored table won't work" << endl;
			exit(0);
			}
		else if (s == "-nohyperlinks")
			{
			gH2L.setAllowHyperlinks(false);
			}
		else if (s == "-spiderlimit")
			{
			if (++x >= argc) continue;
			spider_limit = atoi(argv[x]);
			}
		else if (s == "-allowfilefetch")
			{
			gH2L.setAllowFileFetch(true);
			}
		else if (s == "-data")
			{
			if (++x >= argc) continue;
			gH2L.setPathToDataFile(string(argv[x]));
			}
		else if (s == "-v")
			{
			cout << "Version 1.15" << endl;
			exit(0);
			}
		else if (s == "-nowhitecolortables")
			{
			gH2L.setColorWhiteTables(false);
			}
		else if (s == "-tmp")
			{
			if (++x >= argc) continue;
			gH2L.setTempPath(stripstring(string(argv[x])));
			}
		else if (s == "-")
			{
			nostdin = true;
			}
		else if (s == "-paperwidth")
			{
			if (++x >= argc) continue;
			gH2L.setPaperWidth(argv[x]);
			}
		else if (s == "-paperheight")
			{
			if (++x >= argc) continue;
			gH2L.setPaperHeight(argv[x]);
			}
		else if (s == "-hmargin")
			{
			if (++x >= argc) continue;
			gH2L.setHorizontalMargin(argv[x]);
			}
		else if (s == "-vmargin")
			{
			if (++x >= argc) continue;
			gH2L.setVerticalMargin(argv[x]);
			}
		else if (s == "-nostyles")
			{
			gH2L.setProcessStyles(false);
			}
		else if (s == "-notables")
			{	
			gH2L.setProcessTables(false);
			}
		else if (s == "-inlinetables")
			{
			gH2L.setInlineTables(true);
			}
		else if (s == "-nospider")
			{
			nospider = true;
			}
		else if (s == "-noimages")
			{
			gH2L.setProcessImages(false);
			}
		else if (s == "-url")
			{
			if (++x >= argc) continue;
			url = argv[x];
			split_url(url,server,path,file);
			gH2L.setURLServerRoot(server);
			gH2L.setURLCurrentPath(path);
			gH2L.setURLFileName(file);
			gH2L.setCurrentSpiderTag(gH2L.add_to_spider_table(server+path+file,"",true));
			}
		else 
			{
			cerr << "Unknown option [" << s << "]" << endl;
			}
		}

	std::string nexturl,nexttag;

	if (!gH2L.openTableDimensionsFile())
		{
		cerr << "Could not open temp file for table dimensions" << endl;
		exit(-1);
		}

	if (nostdin)
		{
		if (lowerstring(stripstring(url)).substr(0,7) == "file://" && (!gH2L.getAllowFileFetch()))
			url = "http://nothing";

		cerr << "Fetching " << url << endl;
		if (newfile)
			pclose(newfile);
#ifdef LINUX
		std::string command;
		if (url.find("file://") == 0)
			{
			command = string("cat ") + url.substr(7,url.size() - 7);
			}
		else
			{
			command=string("wget -q -T 30 -O - ") + "\"" + url + "\"";
			}
		newfile = popen(command.c_str(),"r");
#else
		std::string base_command = "fetch -T 30 -o - ";
		newfile = popen(string(base_command + "\"" + url + "\"").c_str(),"r");
#endif
		yyrestart(newfile);
		}

	do {
		do
			{
			yyparse();
			}
		while(!feof(yyin));

		if (gMainDataStart) 
			{ 
			gH2L.html_close(); 
			gMainDataStart = false; 
			}
		if (nospider) break;

		if (spider_limit == 0)
			break;
		if (spider_limit > 0)
			spider_limit--;

		gH2L.dump_spider_table();

		gH2L.next_spider_url(nexturl,nexttag);
		if (nexturl != "" && nexttag != "")
			{
			std::string new_server,new_path,new_file;
			split_url(nexturl,new_server,new_path,new_file);
			gH2L.setURLServerRoot(new_server);
			gH2L.setURLCurrentPath(new_path);
			gH2L.setURLFileName(new_file);
			gH2L.setCurrentSpiderTag(nexttag);

			cerr << "Fetching " << nexturl << endl;
			if (lowerstring(stripstring(nexturl)).substr(0,7) == "file://" && (!gH2L.getAllowFileFetch()))
				nexturl = "http://nothing";

			if (newfile)
				pclose(newfile);


#ifdef LINUX
			std::string command;
			if (nexturl.find("file://") == 0)
				{
				command = string("cat ") + nexturl.substr(7,nexturl.size() - 7);
				}
			else
				{
				command=string("wget -q -T 30 -O - ") + "\"" + nexturl + "\"";
				}
			newfile = popen(command.c_str(),"r");
#else
			std::string base_command = "fetch -T 30 -o - ";
			newfile = popen(string(base_command + "\"" + nexturl + "\"").c_str(),"r");
#endif
			gH2L.setSpidering(true);
			yyrestart(newfile);
			}
		}
	while (nexttag != "");

	gH2L.setSpidering(false);
	if (gMainDataStart) 
		{ 
		gH2L.html_close(); 
		gMainDataStart = false; 
		}

	gH2L.cleanup(); 
	gH2L.generate_latex_conclusion();
	gH2L.closeTableDimensionsFile();
}


