/*
 * This file is part of htmltolatex (h2l) a HTML to Latex converter
 * Copyright (C) 2001 Neil Radisch (nradisch@panix.com)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file copying.txt.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */
#ifndef	SAFE_OSTRSTREAM_H_
#define SAFE_OSTRSTREAM_H_

//////////////////////////////////////////////////////////////////////////////
// safe_ostrstream
//
// The string stream can be very useful, but having to delete the result
// of dynamic usage can cause alot of headaches, particularly if exceptions
// are thrown. This is a trivial subclass that cleans up after itself.
/////////////////////////////////////////////////////////////////////////////

#ifdef __GNUC__
#include <strstream>

// Put this in a class to disable class copying
#define	CLASS_NOCOPY(x) \
private: \
	x(const x &); \
	x &operator=(const x &); \

class safe_ostrstream: public ostrstream
{
private:
	char	*mResult;

	CLASS_NOCOPY(safe_ostrstream)

public:
	safe_ostrstream();
	~safe_ostrstream();
	const char *str();

};

#else

#include	<string>
#include	<sstream>
#include	"nmr_nocopy.h"

class safe_ostrstream: public std::ostringstream
{
	CLASS_NOCOPY(safe_ostrstream)

private:
	std::string	mResult;

public:
	safe_ostrstream(){}
	~safe_ostrstream(){}
	const char *str()
		{
		using namespace std;
		mResult = ostringstream::str();
		return mResult.c_str();
		}
};


#endif  //__GNUC__ defined

#endif //SAFE_OSTRSTREAM_H_





